import requests, json
import os.path
import re
import urllib.parse
import csv
from typing import Dict, Tuple
import pandas as pd
import datetime
from dateutil import parser

import regex_patterns

# Constants for the authentication and query process
DEFAULT_AUTH_URL = 'https://app.leak.center/uaa/oauth/token'
DEFAULT_LEAKS_ENDPOINT = "https://app.leak.center/svc-saas/leak/search"
DEFAULT_CLIENT_ID = 'client-api'
DEFAULT_CLIENT_SECRET = 'comfy-litigate-embargo-forelimb'
DEFAULT_DEBUG_REGEX = False
DEFAULT_TAGS = ['account', 'password', 'username']

# Load environment variables if available
def load_env_if_available():
    try:
        from dotenv import load_dotenv
        # Check if .env file exists before trying to load it
        if os.path.isfile('.env'):
            print("Found .env file. Do you want to use credentials from .env? (Y/n): ", end='')
            use_env = input().strip().lower()
            if use_env == '' or use_env == 'y' or use_env == 'yes':
                load_dotenv()
                print("Environment variables loaded from .env file.")
                return True
            else:
                print("Not using .env file.")
        return False
    except ImportError:
        print("python-dotenv not installed, not using environment variables.")
        return False

# Variable to track if env is loaded
env_loaded = False

# Default values from environment
DEFAULT_DOMAIN = os.getenv('DEFAULT_DOMAIN', '')


class Auth:
    username = None
    password = None


class ProcessLeaks:
    """
    """
    def __init__(self, domain: str, token: str, date_range: str, tags: list, domain_filter: bool = False, leak_id: str = None):
        self.domain = domain
        self.token = token
        self.date_range = date_range
        self.tags = tags
        self.leak_id = leak_id
        self.is_debug = False
        self.is_debug_regex = DEFAULT_DEBUG_REGEX
        self._output_folder = 'output_files'
        self._empty_string = 'N/A'
        self._domain_filter = domain_filter
        
        # Base filename without extension
        base_name = self._get_output_base_filename(domain)
        
        # If domain filter is enabled, we'll create two separate files
        if self._domain_filter:
            self._file_domain_emails = f"{base_name}_domain_emails.csv"
            self._file_external_emails = f"{base_name}_external_emails.csv"
            self._file = None  # No combined file when filtering is on
        else:
            self._file = f"{base_name}.csv"
            self._file_domain_emails = None
            self._file_external_emails = None
            
        self._file_debug = f"debug_files/output_data_{self.domain}.json"
        self._init()

    def _get_output_base_filename(self, domain):
        """
        Generate a base filename for output, using versioning if file already exists
        """
        base_filename = f"{self._output_folder}/output_data_{domain}"
        
        # Check if any files with this base name exist (with any extension)
        existing_files = [f for f in os.listdir(self._output_folder) if f.startswith(f"output_data_{domain}")]
        
        if not existing_files:
            return base_filename
        
        # File exists, create a versioned filename
        version = 2
        while True:
            versioned_base = f"{self._output_folder}/output_data_{domain}_v{version}"
            existing_versioned = [f for f in os.listdir(self._output_folder) 
                             if f.startswith(f"output_data_{domain}_v{version}")]
            
            if not existing_versioned:
                return versioned_base
            version += 1

    def _init(self):
        if not os.path.isdir(self._output_folder):
            os.makedirs(self._output_folder)

        if not os.path.isdir('debug_files'):
            os.makedirs('debug_files')

    def run(self):
        if self.is_debug_regex:
            self._debug_regex()
            return

        count_all_results = self._run()
        if self._domain_filter:
            self._remove_duplicates_filtered_files()
        else:
            self._remove_duplicates()
        return count_all_results

    def _run(self):
        debug_results = []
        page = 0
        count_all_results = 0
        
        # Define the base columns and additional columns
        base_columns = ['URL', 'User', 'Password']
        additional_columns = ['IP', 'PC', 'LoggedInUser', 'Time', 'SessionCookie', 'OtherCookies']
        metadata_columns = ['leakPublishDate', 'fileName']
        
        # Initialize file writers with all columns
        if self._domain_filter:
            domain_file = open(self._file_domain_emails, mode='w')
            external_file = open(self._file_external_emails, mode='w')
            
            domain_writer = csv.writer(domain_file, delimiter=';', quotechar='"', quoting=csv.QUOTE_MINIMAL)
            external_writer = csv.writer(external_file, delimiter=';', quotechar='"', quoting=csv.QUOTE_MINIMAL)
            
            # Write headers with all columns
            all_columns = base_columns + additional_columns + metadata_columns
            domain_writer.writerow(all_columns)
            external_writer.writerow(all_columns)
        else:
            combined_file = open(self._file, mode='w')
            combined_writer = csv.writer(combined_file, delimiter=';', quotechar='"', quoting=csv.QUOTE_MINIMAL)
            combined_writer.writerow(base_columns + additional_columns + metadata_columns)

        try:
            while True:
                params = self._get_query_params(page)
                data = self._send_request(params)
                
                # Debug: Print the number of results received from API
                print(f"API returned {len(data['content'])} results")
                
                filtered_content = filter_results(data['content'], self.tags, self.domain)
                
                # Debug: Print the number of filtered results
                print(f"After filtering: {len(filtered_content)} results")
                
                if self.is_debug:
                    debug_results.extend(data['content'])

                # Debug: Let's examine the first few filtered items
                if filtered_content and page == 0:
                    print("\nDebug - Sample of filtered content:")
                    for i, item in enumerate(filtered_content[:3]):  # Print first 3 items
                        print(f"Item {i+1}:")
                        for k, v in item.items():
                            if k == 'LeakContent':
                                print(f"  {k}: {v[:100]}..." if len(v) > 100 else f"  {k}: {v}")
                            else:
                                print(f"  {k}: {v}")
                    print()

                domain_count = 0
                external_count = 0
                
                for key, item in enumerate(filtered_content):
                    # Get the base row data
                    base_row = self._get_row(item)
                    
                    # Extract additional fields from the content
                    additional_fields = self._extract_additional_fields(item.get('LeakContent', ''))
                    
                    # Create the complete row as a list
                    row = [
                        base_row[0],  # URL
                        base_row[1],  # User
                        base_row[2],  # Password
                        additional_fields.get('IP', 'N/A'),
                        additional_fields.get('PC', 'N/A'),
                        additional_fields.get('LoggedInUser', 'N/A'),
                        additional_fields.get('Time', 'N/A'),
                        additional_fields.get('SessionCookie', 'N/A'),
                        additional_fields.get('OtherCookies', 'N/A'),
                        base_row[3],  # leakPublishDate
                        base_row[4]   # fileName
                    ]
                    
                    if self._domain_filter:
                        user_email = row[1]  # User email is at index 1
                        
                        # Check if user email contains the target domain
                        if user_email and f"@{self.domain}" in user_email.lower():
                            domain_writer.writerow(row)
                            domain_count += 1
                        else:
                            external_writer.writerow(row)
                            external_count += 1
                    else:
                        combined_writer.writerow(row)

                # Debug: Print counts for this page
                if self._domain_filter:
                    print(f"Page {page + 1}: Added {domain_count} domain emails and {external_count} external emails")
                
                count_all_results += len(filtered_content)
                print(f"Page {page + 1} processed, total entries collected: {count_all_results}")

                if data['last']:
                    break

                if self.is_debug:
                    if page >= 0:
                        break

                page += 1
        finally:
            # Close file handles
            if self._domain_filter:
                domain_file.close()
                external_file.close()
            else:
                combined_file.close()

        if self.is_debug:
            with open(self._file_debug, 'w') as f:
                json.dump(debug_results, f, indent=4, sort_keys=True)

        return count_all_results

    def _get_query_params(self, page):
        # Always start with the domain
        query = self.domain
        
        # Add leak ID if provided (using AND operator)
        if self.leak_id:
            query += f" AND leakId:{self.leak_id}"
            
        # Add date range if provided (using AND operator)
        if self.date_range:
            query += f" AND createdAt:[{self.date_range}]"
            
        params = {
            "page": page,
            "size": 1000,
            "sort": "createdAt,desc",
            "length": 500,
            "highlight": "true",
            "query": query
        }
        return params

    def _remove_duplicates(self):
        """
        Remove duplicates in csv file base on columns in "subset" defined
        :return:
        """
        print("Start removing duplicates...")
        df = pd.read_csv(self._file, delimiter=';', keep_default_na=False)
        df.drop_duplicates(keep='first', inplace=True, subset=['URL', 'User', 'Password'])
        print("Updating the file...")
        df.to_csv(self._file, sep=";", index=False)
        print("Duplicates has been removed")
    
    def _remove_duplicates_filtered_files(self):
        """
        Remove duplicates in both filtered CSV files
        """
        print("Start removing duplicates from domain emails file...")
        if os.path.exists(self._file_domain_emails) and os.path.getsize(self._file_domain_emails) > 0:
            df_domain = pd.read_csv(self._file_domain_emails, delimiter=';', keep_default_na=False)
            df_domain.drop_duplicates(keep='first', inplace=True, subset=['URL', 'User', 'Password'])
            print("Updating domain emails file...")
            df_domain.to_csv(self._file_domain_emails, sep=";", index=False)
            print(f"Found {len(df_domain)} unique domain email entries")
        else:
            print("No domain email entries found")
            
        print("Start removing duplicates from external emails file...")
        if os.path.exists(self._file_external_emails) and os.path.getsize(self._file_external_emails) > 0:
            df_external = pd.read_csv(self._file_external_emails, delimiter=';', keep_default_na=False)
            df_external.drop_duplicates(keep='first', inplace=True, subset=['URL', 'User', 'Password'])
            print("Updating external emails file...")
            df_external.to_csv(self._file_external_emails, sep=";", index=False)
            print(f"Found {len(df_external)} unique external email entries")
        else:
            print("No external email entries found")
            
        print("Duplicates have been removed from both files")

    def _send_request(self, params: Dict):
        """
        Send request to API
        """
        global LEAKS_ENDPOINT
        
        headers = {
            'Authorization': f'Bearer {self.token}'
        }
        url = f"{LEAKS_ENDPOINT}?{urllib.parse.urlencode(params)}"
        
        # Print the full query URL for debugging and transparency
        print(f"Query URL: {url}")
        
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        return response.json()

    def _get_row(self, item: Dict) -> tuple[str, str, str, str, str]:
        """
        Formatting csv row
        """
        url_title = self._empty_string
        user_title = ''
        pass_title = ''
        pub_date_title = ''
        file_name = ''

        for key, value in item.items():
            if key == 'Server':
                # Clean the URL by removing the additional fields
                if value and '|' in value:
                    # Only keep the part before the first pipe
                    url_title = value.split('|')[0]
                else:
                    url_title = value if value else self._empty_string
            if key == 'User':
                user_title = self.clear_email(value)
            if key == 'Password':
                pass_title = value
            if key == 'createdAt':
                pub_date_title = value
            if key == 'fileName':
                file_name = value
        row = (url_title, user_title, pass_title, pub_date_title, file_name)
        return row

    def _debug_regex(self):
        data = {}
        data['content'] = [
            {
                "content": "ia.speedantre/;smkn1tab10otkp3;qwerty\nhttps://accounts.google.com/;kalyanisjunghari@gmail.com;Kalyani@2003\nhttps://sukienbts-freefire.vn/;PhuongNguyendnv@gmail.com;[NOT_SAVED]\nhttps://www.facebook.com;Trc Dung;EmGhetAnh80\nhttps://<kaduu:h>guest.buhlergroup.com</kaduu:h>/;gustavo.saito@buhlergroup.com;288341\nhttps://cls6.edunet.net/cyber/cm/mcom/pmco000b00.do;sje20346;sje20346!\nhttps://discord.com/login;arthurjmj274@gmail.com;97426394\nhttp://192.168.178.1/;;hold8605\nhttps://idp.omlet.me/register;ProPlayer1234;wr@Xbm",
                "createdAt": "2025-02-04 13:06:46",
                "cvssScore": 7.0,
                "fileExtension": "txt",
                "fileName": "stelear logs 01.02.2025/2025.02.012318freelogpassforallcloudsbase251.txt",
                "id": "1785566a-d0aa-3352-b6d1-ae151e885c7a",
                "leakDiscoverDate": "2025-02-03",
                "leakId": "ca0856e8-50be-398a-8119-612e3be65a11",
                "leakName": "stelear logs 01.02.2025",
                "leakPublishDate": "2025-02-01",
                "leakSize": 5408049582,
                "leakSource": "darknet",
                "leakTags": "account"
            }
        ]
        filtered_content = filter_results(data['content'], self.tags, self.domain)
        for key, item in enumerate(filtered_content):
            row = self._get_row(item)
            print('--row--')
            print(row)

    def clear_email(self, email):
        match = re.search(regex_patterns.get_email_group_pattern(), email, re.IGNORECASE)
        if match:
            result = match.groupdict()
            email = result.get('Email')

        return email

    def _extract_additional_fields(self, content):
        """
        Extract additional fields from the content string
        """
        fields = {}
        
        # Handle the case where no pipe separators are present
        if '|' not in content:
            return fields
            
        # Split by | to get individual field-value pairs
        parts = content.split('|')
        
        # Skip the first part (which is the base URL:user:password)
        for part in parts[1:]:  # Start from the second part
            # Skip empty parts
            if not part.strip():
                continue
                
            # Split by : to get field and value
            field_parts = part.split(':', 1)
            if len(field_parts) == 2:
                field = field_parts[0].strip()
                value = field_parts[1].strip()
                
                # Map the field to our additional columns
                if field in ['IP', 'PC', 'LoggedInUser', 'Time', 'SessionCookie', 'OtherCookies']:
                    fields[field] = value
        
        return fields

    def extract_content_parts(self, content):
        """
        Split the content into base part and additional fields part
        Returns a tuple of (base_content, additional_fields_content)
        """
        if '|' not in content:
            return content, ""
            
        parts = content.split('|', 1)
        return parts[0], '|' + parts[1] if len(parts) > 1 else ""


def get_credentials():
    if Auth.username and Auth.password:
        return Auth.username, Auth.password
    
    # Try to get credentials from environment variables if env was loaded
    if env_loaded:
        env_username = os.getenv('LEAK_CENTER_USERNAME')
        env_password = os.getenv('LEAK_CENTER_PASSWORD')
        
        if env_username and env_password:
            print(f"Using credentials from .env file: {env_username}")
            return env_username, env_password
    
    # Fall back to auth.txt file
    try:
        with open("auth.txt", "r") as f:
            creds = f.read().strip().split(',')
            print(f"Using credentials from auth.txt file: {creds[0]}")
            return creds
    except FileNotFoundError:
        raise ValueError("No credentials found. Please create an auth.txt file or use .env file.")


def get_access_token(username, password):
    """
    Get access token for API
    """
    global AUTH_URL, CLIENT_ID, CLIENT_SECRET
    
    print(f'Authenticating as {username}...')
    data = {
        'grant_type': 'password',
        'username': username,
        'password': password,
        'client_id': CLIENT_ID,
        'client_secret': CLIENT_SECRET
    }
    response = requests.post(AUTH_URL, data=data)
    response.raise_for_status()
    token = response.json()['access_token']
    print(f"Success Authentication, Fetched token: {token[:10]}...")
    return token


def get_query_input():
    # Try to get domain from environment variable first
    env_domain = os.getenv('DEFAULT_DOMAIN')
    if env_domain:
        return env_domain
        
    # Fall back to input.txt file
    try:
        with open("input.txt", "r") as f:
            return f.read().strip()
    except FileNotFoundError:
        return None


def get_date_range():
    try:
        with open("DateRange.txt", "r") as f:
            raw_date_range = f.read().strip()
            return parse_date_range(raw_date_range)
    except FileNotFoundError:
        return ""  # This should be replaced with actual in-memory variable if available


def parse_date_range(date_range_str):
    """
    Parse various date range formats and convert to the API format: 'YYYY-MM-DD TO YYYY-MM-DD'
    or 'YYYY-MM-DD TO *' for open-ended ranges.
    
    Supported formats:
    - YYYY-MM-DD TO YYYY-MM-DD
    - YYYY-MM-DD TO *
    - YYYY-YYYY
    - Month DD TO Month DD
    - DD.MM.YY-DD.MM.YY
    - DD.MM.YYYY to DD.MM.YYYY
    """
    if not date_range_str:
        return ""
    
    # If already in the correct format, return as is
    if re.match(r'^\d{4}-\d{2}-\d{2} TO (\d{4}-\d{2}-\d{2}|\*)$', date_range_str):
        return date_range_str

    # Handle special case: YYYY-YYYY format (year range)
    year_range_match = re.match(r'^(\d{4})-(\d{4})$', date_range_str)
    if year_range_match:
        start_year = year_range_match.group(1)
        end_year = year_range_match.group(2)
        return f"{start_year}-01-01 TO {end_year}-12-31"
    
    # Split the range by common separators
    separators = [' to ', ' TO ', '-', ' - ', ' – ', '–', ' — ', '—']
    start_date = None
    end_date = None
    
    for separator in separators:
        if separator in date_range_str:
            parts = date_range_str.split(separator, 1)
            try:
                # Handle open-ended ranges
                if parts[1].strip() in ['*', 'now', 'present', 'today']:
                    start_date = parser.parse(parts[0], dayfirst=True, fuzzy=True)
                    return f"{start_date.strftime('%Y-%m-%d')} TO *"
                
                # Parse both parts
                start_date = parser.parse(parts[0], dayfirst=True, fuzzy=True)
                end_date = parser.parse(parts[1], dayfirst=True, fuzzy=True)
                
                # Format in required format
                return f"{start_date.strftime('%Y-%m-%d')} TO {end_date.strftime('%Y-%m-%d')}"
            except (ValueError, parser.ParserError) as e:
                print(f"Error parsing date range '{date_range_str}': {e}")
                continue
    
    # If we reach here, try to interpret the entire string as a single date (today - that date)
    try:
        single_date = parser.parse(date_range_str, dayfirst=True, fuzzy=True)
        today = datetime.datetime.now()
        return f"{single_date.strftime('%Y-%m-%d')} TO {today.strftime('%Y-%m-%d')}"
    except (ValueError, parser.ParserError):
        print(f"Could not parse date range: {date_range_str}")
        return ""


def get_tags():
    # Try to get tags from environment variable if env was loaded
    if env_loaded:
        env_tags = os.getenv('DEFAULT_TAGS')
        if env_tags:
            tags = env_tags.split(',')
            print(f"Using tags from .env file: {', '.join(tags)}")
            return tags
    
    # Fall back to tags.txt file
    try:
        with open("tags.txt", "r") as f:
            tags = [line.strip() for line in f]
            print(f"Using tags from tags.txt file: {', '.join(tags)}")
            return tags
    except FileNotFoundError:
        print("No tags.txt file found. Using default tags.")
        return DEFAULT_TAGS


def get_domain_filter_setting():
    """
    Check if domain filtering is enabled in filter.txt
    """
    try:
        with open("filter.txt", "r") as f:
            settings = f.readlines()
            for line in settings:
                if line.strip().lower().startswith("domainfilter:"):
                    setting = line.split(":", 1)[1].strip().lower()
                    return setting in ["on", "true", "yes", "1"]
    except FileNotFoundError:
        return None  # File doesn't exist, will prompt user


def get_leak_id():
    """
    Get leak ID from leakid.txt if it exists, otherwise ask the user
    """
    leak_id_from_file = None
    try:
        with open("leakid.txt", "r") as f:
            leak_id_from_file = f.read().strip()
    except FileNotFoundError:
        pass
    
    # If file doesn't exist or is empty, ask the user
    if not leak_id_from_file:
        leak_id = input('Enter leak ID (or press Enter to skip): ')
        if leak_id:
            try:
                with open("leakid.txt", "w") as f:
                    f.write(leak_id)
                print(f"Leak ID '{leak_id}' saved to leakid.txt for future use.")
            except Exception as e:
                print(f"Note: Could not save leak ID to leakid.txt: {e}")
        return leak_id
    else:
        # If leak ID exists in file, ask if user wants to use it
        print(f"Leak ID found in leakid.txt: {leak_id_from_file}")
        confirm = input(f"Do you want to use this leak ID? (Y/n): ").strip().lower()
        
        if confirm in ['', 'y', 'yes']:
            return leak_id_from_file
        else:
            # User doesn't want to use the stored leak ID
            leak_id = input('Enter new leak ID (or press Enter to skip): ')
            if leak_id:
                try:
                    with open("leakid.txt", "w") as f:
                        f.write(leak_id)
                    print(f"New leak ID '{leak_id}' saved to leakid.txt.")
                except Exception as e:
                    print(f"Note: Could not save leak ID to leakid.txt: {e}")
            else:
                # User entered empty string, clear the leakid.txt file
                try:
                    with open("leakid.txt", "w") as f:
                        f.write("")
                    print("Leak ID cleared.")
                except Exception as e:
                    print(f"Note: Could not clear leak ID in leakid.txt: {e}")
            return leak_id


def query_and_process_leaks(domain, token, date_range, tags, domain_filter=False, leak_id=None):
    """
    """
    process_leaks = ProcessLeaks(domain, token, date_range, tags, domain_filter, leak_id)
    count_results = process_leaks.run()
    return count_results


def is_valid_server(server):
    """
    Check if the server string is a valid URL format.
    """
    valid_server_pattern = re.compile(
        r"^(https?://|http://|android://|ftp://|ttps://|ttp://|tps://|tp://|ps://|s://|p://|oid://)?([a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)+)(:[0-9]+)?(/[^\s]*)?$")
    return bool(valid_server_pattern.match(server))


def extract_info(input_string, domain):
    # First check if additional fields are present, and if so, strip them off
    if '|' in input_string:
        input_string = input_string.split('|')[0]
    
    for pattern in regex_patterns.get_patterns(domain):
        match = re.search(pattern, input_string)
        if match:
            if DEFAULT_DEBUG_REGEX:
                print('---Domain---', domain)
                print('---Input String---', input_string)
                print('---Pattern---', pattern)
            result = match.groupdict()
            if result.get('Server') and not is_valid_server(result['Server']):
                result['Server'] = None
            if result.get('Server'):
                result['Server'] = re.sub(r'^(ttps|ttp|tps|tp|ps|s|p|oid)://', '', result['Server'])
            if result.get('User') and 'https://' in result.get('User'):
                result['User'] = None

            extracted_info = dict()
            for key in ['Server', 'User', 'Password']:
                value = result.get(key, "")
                if value is None or value.lower() in ["none", ":", ";", ",", "null", "empty", "unknown"]:
                    extracted_info[key] = ""
                else:
                    extracted_info[key] = value

            return extracted_info

    return {'Server': "", 'User': "", 'Password': ""}


def filter_results(results, tags, domain):
    filtered_content = []
    # Use re.IGNORECASE for case-insensitive matching
    # This pattern will match if ANY of the tags are found (not all of them)
    tag_pattern = re.compile("|".join(tags), re.IGNORECASE)
    
    # Dictionary to track skip counts for similar leaks
    skip_counts = {}
    
    # Process all results
    for result in results:
        try:
            leak_tags = result.get('leakTags', '')
            tag_found = False
            if leak_tags:
                tag_found = tag_pattern.search(leak_tags)
        except Exception:
            tag_found = False
            
        if tag_found:
            content_lines = result['content'].split('\n')
            for line in content_lines:
                if "<kaduu:h>" in line:
                    line = cut_sql_content(line)

                    clean_line = re.sub(r'<kaduu:h>|</kaduu:h>|\\|^\s', '', line)  # fix1 |^\s
                    data = {
                        "LeakContent": clean_line,
                        "createdAt": result['createdAt'],
                        "fileName": result['fileName'],
                        # "leakSource": result['leakSource'],
                        # "leakSize": result['leakSize'],
                        # "leakPublishDate": result['leakPublishDate'],
                        # "leakTags": result['leakTags']
                    }
                    
                    # Extract just the base URL:user:password part for the regular extraction
                    base_content = clean_line
                    if '|' in clean_line:
                        base_content = clean_line.split('|')[0]
                    
                    info = extract_info(base_content, domain)
                    
                    for key, value in info.items():
                        if value:
                            data[key] = value
                    
                    filtered_content.append(data)
        else:
            # Get relevant leak information for debugging
            leak_name = result.get('leakName', 'Unknown')
            leak_tags = result.get('leakTags', 'None')
            leak_id = result.get('leakId', 'Unknown')
            leak_size = result.get('leakSize', 0)
            leak_size_mb = round(leak_size / (1024 * 1024), 2) if leak_size else 0
            
            # Create a key for this type of leak
            skip_key = f"{leak_name}:{leak_tags}"
            if skip_key in skip_counts:
                skip_counts[skip_key]['count'] += 1
            else:
                skip_counts[skip_key] = {
                    'name': leak_name,
                    'tags': leak_tags,
                    'size_mb': leak_size_mb,
                    'count': 1
                }
    
    # Report skipped leaks in a more concise format
    if skip_counts:
        print("\n--- Skipped Leaks Summary ---")
        for key, info in skip_counts.items():
            leak_name = info['name'] if info['name'] not in ['Unknown', 'None'] else 'Unnamed leak'
            leak_size = f"{info['size_mb']} MB" if info['size_mb'] > 0 else 'unknown size'
            tags_info = f"tags: {info['tags']}" if info['tags'] not in ['None', None, ''] else 'no tags'
            
            print(f"• {leak_name} ({leak_size}) - {tags_info} - skipped {info['count']} times")
        
        print(f"Total unique leak types skipped: {len(skip_counts)}")
        print(f"Required tags for matching (case-insensitive): {', '.join(tags)}")
        print("----------------------------\n")
    
    return filtered_content


def cut_sql_content(line):  # todo to regex
    match = re.search("\),\(", line)
    if match:
        sql_lines = line.split("),(")
        for sql_line in sql_lines:
            if "<kaduu:h>" in sql_line:
                line = sql_line

    match = re.search("\\\\r\\\\", line)
    if match:
        sql_lines = line.split("\\r\\")
        for sql_line in sql_lines:
            if "<kaduu:h>" in sql_line:
                line = sql_line
    return line


def main():
    global AUTH_URL, LEAKS_ENDPOINT, CLIENT_ID, CLIENT_SECRET, IS_DEBUG_REGEX, env_loaded
    
    # Ask to load environment variables if available
    env_loaded = load_env_if_available()
    
    # Set config variables based on env or defaults
    AUTH_URL = os.getenv('LEAK_CENTER_AUTH_URL', DEFAULT_AUTH_URL) if env_loaded else DEFAULT_AUTH_URL
    LEAKS_ENDPOINT = os.getenv('LEAK_CENTER_LEAKS_ENDPOINT', DEFAULT_LEAKS_ENDPOINT) if env_loaded else DEFAULT_LEAKS_ENDPOINT
    CLIENT_ID = os.getenv('LEAK_CENTER_CLIENT_ID', DEFAULT_CLIENT_ID) if env_loaded else DEFAULT_CLIENT_ID
    CLIENT_SECRET = os.getenv('LEAK_CENTER_CLIENT_SECRET', DEFAULT_CLIENT_SECRET) if env_loaded else DEFAULT_CLIENT_SECRET
    IS_DEBUG_REGEX = os.getenv('DEBUG_REGEX', 'false').lower() == 'true' if env_loaded else DEFAULT_DEBUG_REGEX
    
    # Authentication process
    token = None
    auth_succeeded = False
    max_retries = 3
    retry_count = 0
    
    while not auth_succeeded and retry_count < max_retries:
        try:
            if retry_count == 0:
                # First try with credentials from env or auth.txt
                try:
                    username, password = get_credentials()
                except ValueError as e:
                    print(str(e))
                    username = input('Please enter your username (email): ')
                    password = input('Enter your password: ')
            else:
                # Subsequent retries, always prompt for credentials
                print(f"\nAuthentication failed. Retry {retry_count} of {max_retries-1}")
                username = input('Please enter your username (email): ')
                password = input('Enter your password: ')

            if not username or not password:
                print('Username and password are required values')
                retry_count += 1
                continue
            
            token = get_access_token(username, password)
            auth_succeeded = True
            
            # Save working credentials to auth.txt if they aren't there already
            if retry_count > 0 and not env_loaded:
                try:
                    with open("auth.txt", "w") as f:
                        f.write(f"{username},{password}")
                    print("Credentials saved to auth.txt for future use.")
                except Exception as e:
                    print(f"Note: Could not save credentials to auth.txt: {e}")
            
        except Exception as e:
            print(f'Authentication failed: {str(e)}')
            retry_count += 1
    
    if not auth_succeeded:
        print("Maximum authentication attempts reached. Exiting.")
        return

    # Get domain from input.txt
    domain_from_file = get_query_input()

    # Display and confirm the domain
    if domain_from_file:
        print(f"Domain found in input.txt: {domain_from_file}")
        confirm = input(f"Do you want to query this domain? (Y/n): ").strip().lower()
        
        if confirm in ['', 'y', 'yes']:
            domain = domain_from_file
        else:
            domain = input('Enter your domain like example.com (no subdomain): ')
    else:
        domain = input('No domain found in input.txt. Enter your domain like example.com (no subdomain): ')

    if not domain:
        print('Domain is empty')
        return
    
    # Save the domain to input.txt if it's different from what was there
    if domain != domain_from_file:
        try:
            with open("input.txt", "w") as f:
                f.write(domain)
            print(f"Domain '{domain}' saved to input.txt for future use.")
        except Exception as e:
            print(f"Note: Could not save domain to input.txt: {e}")

    # Get leak ID (if available)
    leak_id = get_leak_id()
    if leak_id:
        print(f"Using leak ID: {leak_id}")

    # Check for domain filtering preference
    domain_filter = get_domain_filter_setting()
    if domain_filter is None:  # Not set in file, ask user
        filter_prompt = input(f"Do you want to create separate files for accounts with '{domain}' emails vs external emails? (y/N): ").strip().lower()
        domain_filter = filter_prompt in ['y', 'yes']
        
        # Save the preference for future use
        try:
            with open("filter.txt", "w") as f:
                f.write(f"domainfilter:{'on' if domain_filter else 'off'}")
            print(f"Filter preference saved to filter.txt")
        except Exception as e:
            print(f"Note: Could not save filter preference to filter.txt: {e}")
    
    date_range = get_date_range()

    if not date_range:
        # format 2024-01-01 TO *
        raw_date_range = input('Enter your date range (e.g., 2024-02-01 TO *, 2024-02-01 TO 2025-01-31, 2022-2023, may 21 to june 23, 23.1.24-30.5.24) or press enter to ignore date range: ')
        date_range = parse_date_range(raw_date_range)
        if raw_date_range and not date_range:
            print(f"Warning: Could not parse date range '{raw_date_range}', proceeding without date filter.")

    try:
        tags = get_tags()
    except Exception:
        print("No tags.txt file found. Using default tags.")
        tags = ["account", "password", "username"]

    final_results = query_and_process_leaks(domain, token, date_range, tags, domain_filter, leak_id)

    if domain_filter:
        base_filename = os.path.splitext(os.path.basename(ProcessLeaks(domain, token, date_range, tags, domain_filter, leak_id)._file_domain_emails))[0].replace("_domain_emails", "")
        print(f"Completed. Results saved to:")
        print(f"- {base_filename}_domain_emails.csv (accounts with '{domain}' email addresses)")
        print(f"- {base_filename}_external_emails.csv (accounts with other email domains)")
    else:
        print(f"Completed. Results saved to output_files/output_data_{domain}.csv")


if __name__ == "__main__":
    main() 